﻿//////////////////////////////////////////////
// ProgramManager.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkGraphics
{
	class Program ;
}

/// Includes ---------------------------------

// nkGraphics
#include "../Dll/DllDefines.h"

#include "../Utilities/InfiniteMask.h"

// nkCommon
#include <nilkinsCommon/Patterns/SingletonClass.h>

// Standards
#include <map>
#include <unordered_map>
#include <mutex>
#include <string>
	
/// Class ------------------------------------

namespace nkGraphics
{
	class DLL_GRAPHICS_EXPORT ProgramManager final : public nkCommon::SingletonClass<ProgramManager>
	{
		friend class nkCommon::SingletonClass<ProgramManager> ;

		private :
		
			// Attributs
			// Mutex unique pour tout, à voir si bénéfice à split
			std::mutex _mutex ;

			// Mémoire des shaders instanciés
			std::unordered_map<std::string, Program*> _programMap ;

			// Compteurs pour les ids
			unsigned int _programCounter ;
			std::map<InfiniteByteMask, unsigned int> _knownSemantics ;
			std::unordered_map<std::string, int> _customSemanticsIndex ;

		private :

			// Functions
			// Constructor
			ProgramManager () ;
		
		public :

			// Destructor
			~ProgramManager () ;

			// Manipulating memory
			Program* createOrRetrieve (const std::string_view& name) ;
			Program* get (const std::string_view& name) ;
			Program* getByIndex (unsigned int index) ;
			void rename (const std::string_view& oldName, const std::string_view& newName) ;
			void erase (const std::string_view& name) ;

			// Managing ids for inputs
			unsigned int getIdForInputSemantics (Program* program) ;
			// Bonus semantics (for instances)
			void addCustomSemanticIndex (const std::string_view& semanticName, int assignedIndex) ;
			int getCustomSemanticIndex (const std::string_view& semanticName) ;
	} ;
}