﻿//////////////////////////////////////////////
// RenderQueueManager.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkGraphics
{
	class Entity ;
	class PostProcessPass ;
	class RenderQueue ;
}

/// Includes ---------------------------------

// nkGraphics
#include "../Dll/DllDefines.h"

// Common
#include <nilkinsCommon/Patterns/SingletonClass.h>

// Standards
#include <mutex>
#include <string>
#include <unordered_map>
#include <vector>

/// Class ------------------------------------

namespace nkGraphics
{
	class DLL_GRAPHICS_EXPORT RenderQueueManager final : public nkCommon::SingletonClass<RenderQueueManager>
	{
		friend class nkCommon::SingletonClass<RenderQueueManager> ;

		private :
		
			// Attributs
			// Mutex
			mutable std::recursive_mutex _mutex ;
			// Nos RQ
			std::vector<RenderQueue*> _activeRQ ;
			std::unordered_map<std::string, RenderQueue*> _specialRQ ;

			// La RQ renvoyée lors des demandes pour le rendu
			RenderQueue* _renderingRQ ;

		private :

			// Functions
			// Constructeur
			RenderQueueManager () ;
		
		public :

			// Destructeur
			~RenderQueueManager () ;
			
			// Management
			RenderQueue* createOrRetrieve (int index = -1) ;
			RenderQueue* createOrRetrieve (const std::string_view& name) ;
			RenderQueue* get (unsigned int index) const ;
			RenderQueue* get (const std::string_view& name) const ;
			RenderQueue* getByIndex (unsigned int indexGlobal) const ;
			RenderQueue* getNamedByIndex (unsigned int indexGlobal) const ;
			void rename (const std::string_view& oldName, const std::string_view& newName) ;
			void erase (RenderQueue* queue) ;

			// Pour le rendu
			void updateRenderingQueue (int minRQ, int maxRQ) ;
			void updateRenderingQueue (const std::string_view& name) ;
			void updateRenderingQueue (RenderQueue* fromThisOne) ;
			RenderQueue* getComputedRenderingQueue () const ;
	} ;
}