﻿//////////////////////////////////////////////
// Resource.h
//
// Base pour toute ressource :
// Shader, Mesh, Texture...
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Includes ---------------------------------

// nkGraphics
#include "ResourceLoadState.h"
#include "ResourceType.h"

#include "../Dll/DllDefines.h"

// Standards
#include <string>
	
/// Class ------------------------------------

namespace nkGraphics
{
	class DLL_GRAPHICS_EXPORT Resource
	{
		protected :
		
			// Attributs
			// Etats de chargement
			RESOURCE_LOAD_STATE _loadState ;
			// Le type de la ressource pour l'identifier
			RESOURCE_TYPE _resourceTypeName ;

			// Son chemin
			std::string _resourcePath ;
			// Le nom de la ressource pour l'identifier
			std::string _resourceName ;

			// Si on la cache lors d'un export
			bool _hidden ;
		
		public :
		
			// Constructeur, destructeur
			Resource () ;
			Resource (const char* path) ;
			virtual ~Resource () ;

			// Chargement et déchargement
			virtual bool load () = 0 ;
			virtual void unload () = 0 ;

			// Getter
			bool isReadyForRendering () const ;
			bool isUnloaded () const ;
			RESOURCE_LOAD_STATE getLoadState () const ;
			std::string_view getResourcePath () const ;
			RESOURCE_TYPE getResourceTypeName () const ;
			std::string_view getResourceName () const ;
			bool getHidden () const ;

			// Setters
			virtual void setResourcePath (const std::string_view& path) ;
			void setResourceName (const std::string_view& name) ;
			void setHidden (bool value) ;
	} ;
}