﻿//////////////////////////////////////////////
// StructToString.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkGraphics
{
	enum class PASS_TYPE ;
	enum class RESOURCE_TYPE ;
	enum class TEX_RENDER_FLAG ;
}

/// Includes ---------------------------------

// nkGraphics
#include "DxDefinesWrapper.h"

#include "../Dll/DllDefines.h"

// Standards
#include <string_view>
	
/// Class ------------------------------------

namespace nkGraphics
{
	class DLL_GRAPHICS_EXPORT StructToString
	{
		private :

			// Constructeur, destructeur
			StructToString () = delete ;
			~StructToString () = delete ;

		public :
		
			// Shader type to string
			static const char* getShaderTypeString (SHADER_TYPE val) ;
			static SHADER_TYPE getStringShaderType (const std::string_view& str) ;

			// Shader Semantic
			static const char* getShaderSemanticString (SHADER_SEMANTIC val) ;
			static SHADER_SEMANTIC getStringShaderSemantic (const char* str) ;

			// Program profile
			static const char* getProgramProfileString (PROGRAM_COMPILATION_PROFILE val) ;
			static PROGRAM_COMPILATION_PROFILE getStringProgramProfile (const std::string_view& str) ;

			// Blend
			static const char* getBlendString (BLEND val) ;
			static BLEND getStringBlend (const std::string_view& str) ;
			static BLEND getBlendFromIndex (unsigned int index) ;

			// Blend Op
			static const char* getBlendOpString (BLEND_OP val) ;
			static BLEND_OP getStringBlendOp (const std::string_view& str) ;
			static BLEND_OP getBlendOpFromIndex (unsigned int index) ;

			// Write mask
			static const char* getWriteMaskString (COLOR_WRITE_ENABLE val) ;
			static COLOR_WRITE_ENABLE getStringWriteMask (const std::string_view& str) ;
			static COLOR_WRITE_ENABLE getWriteMaskFromIndex (unsigned int index) ;

			// Comparison func
			static const char* getComparisonFuncString (COMPARISON_FUNC val) ;
			static COMPARISON_FUNC getStringComparisonFunc (const std::string_view& str) ;
			static COMPARISON_FUNC getComparisonFuncFromIndex (unsigned int index) ;

			// Depth write mask
			static const char* getDepthWriteMaskString (DEPTH_WRITE_MASK val) ;
			static DEPTH_WRITE_MASK getStringDepthWriteMask (const std::string_view& str) ;
			static DEPTH_WRITE_MASK getDepthWriteMaskFromIndex (unsigned int index) ;

			// Stencil op
			static const char* getStencilOpString (STENCIL_OP val) ;
			static STENCIL_OP getStringStencilOp (const std::string_view& str) ;
			static STENCIL_OP getStencilOpFromIndex (unsigned int index) ;

			// Fill mode
			static const char* getFillModeString (FILL_MODE val) ;
			static FILL_MODE getStringFillMode (const std::string_view& str) ;
			static FILL_MODE getFillModeFromIndex (unsigned int index) ;

			// Cull Mode
			static const char* getCullModeString (CULL_MODE val) ;
			static CULL_MODE getStringCullMode (const std::string_view& str) ;
			static CULL_MODE getCullModeFromIndex (unsigned int index) ;

			// Address Mode
			static const char* getAddressModeString (TEXTURE_ADDRESS_MODE val) ;
			static TEXTURE_ADDRESS_MODE getStringAddressMode (const std::string_view& str) ;
			static TEXTURE_ADDRESS_MODE getAddressModeFromIndex (unsigned int index) ;

			// Filter
			static const char* getFilteringString (FILTER val) ;
			static FILTER getStringFiltering (const std::string_view& str) ;
			static FILTER getFilteringFromIndex (unsigned int index) ;

			// Usage
			static const char* getUsageString (USAGE val) ;
			static USAGE getStringUsage (const char* str) ;
			static USAGE getUsageFromIndex (unsigned int index) ;

			// Format
			static const char* getFormatString (FORMAT val) ;
			static FORMAT getStringFormat (const std::string_view& str) ;
			static FORMAT getFormatFromIndex (unsigned int index) ;

			// Bind Flag
			static const char* getBindFlagString (BIND_FLAG val) ;
			static BIND_FLAG getStringBindFlag (const char* str) ;
			static BIND_FLAG getBindFlagFromIndex (unsigned int index) ;

			// CPU Access Flag
			static const char* getCpuAccessFlagString (CPU_ACCESS_FLAG val) ;
			static CPU_ACCESS_FLAG getStringCpuAccessFlag (const char* str) ;
			static CPU_ACCESS_FLAG getCpuAccessFlagFromIndex (unsigned int index) ;

			// Misc Flags
			static const char* getMiscFlagString (RESOURCE_MISC_FLAG val) ;
			static RESOURCE_MISC_FLAG getStringMiscFlag (const char* str) ;
			static RESOURCE_MISC_FLAG getMiscFlagFromIndex (unsigned int index) ;

			// Pass type
			static const char* getPassTypeString (PASS_TYPE val) ;
			static PASS_TYPE getStringPassType (const char* str) ;

			// Resource type
			static const char* getResourceTypeString (RESOURCE_TYPE val) ;
			static RESOURCE_TYPE getStringResourceType (const char* str) ;

			// Texture render flag
			static std::string_view getTexRenderFlagString (TEX_RENDER_FLAG val) ;
			static TEX_RENDER_FLAG getStringTexRenderFlag (const std::string_view& str) ;
	} ;
}