﻿//////////////////////////////////////////////
// UserType.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkScripts
{
	class Function ;

	struct ArrayAccessorDescriptor ;
	struct UserTypeFieldDescriptor ;
}

/// Includes ---------------------------------

// nkScripts
#include "../../Dll/DllDefines.h"

#include "../Functions/FunctionDefines.h"

#include "TypeBuiltInFunctions.h"

// nkMemory
#include <NilkinsMemory/Containers/StringView.h>

// Standards
#include <functional>
#include <string>

/// Internals --------------------------------

namespace nkScripts
{
	using ConstructorCallback = std::function<void*(const DataStack&)> ;
	using DestructorCallback = std::function<void(void*)> ;
}

/// Class ------------------------------------

namespace nkScripts
{
	class DLL_SCRIPTS_EXPORT UserType
	{
		protected :

			// Attributes
			// Nom du type
			std::string _name ;
			// Fully qualified dans le namespace ou autre
			std::string _fullName ;

		public :

			// Constructeur, destructeur
			UserType (const nkMemory::StringView& name) ;
			virtual ~UserType () ;

			// Getters
			nkMemory::StringView getName () const ;
			nkMemory::StringView getFullName () const ;
			virtual Function* getMethod (const nkMemory::StringView& name) = 0 ;
			virtual Function* getStaticMethod (const nkMemory::StringView& name) = 0 ;

			// Contrôles sur le user type
			virtual void setConstructor (ConstructorCallback constructor) = 0 ;
			virtual void setDestructor (DestructorCallback destructor) = 0 ;
			virtual Function* addMethod (const nkMemory::StringView& name) = 0 ;
			virtual Function* addStaticMethod (const nkMemory::StringView& name) = 0 ;
			virtual Function* overrideBuiltInFunction (TYPE_BUILT_IN_FUNCTIONS builtInFunc) = 0 ;
			virtual void addField (const UserTypeFieldDescriptor& fieldDescriptor) = 0 ;
			virtual void enableArrayIndexing (const ArrayAccessorDescriptor& descriptor) = 0 ;
			virtual void reset () = 0 ;
	} ;
}