﻿//////////////////////////////////////////////
// Thread.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkTasks
{
	class Scheduler ;
	class Task ;
}

/// Includes ---------------------------------

// nkTasks
#include "../Dll/DllDefines.h"

#include "ThreadId.h"
#include "ThreadSubmittedWork.h"

// Standards
#include <atomic>
#include <condition_variable>
#include <mutex>
#include <queue>
#include <thread>

/// Class ------------------------------------
	
namespace nkTasks
{
	class DLL_TASKS_EXPORT Thread final
	{
		private :
				
			// Fonction du thread qui run
			void asyncProcess () ;

		private :
		
			// Pour le bon déroulement du processus
			// Le thread lui-même
			std::thread _thread ;
			// Mutex de job
			std::mutex _jobMutex ;
			// Condition variable d'attente
			std::condition_variable _conditionWait ;
			// Les jobs qu'on peut nous soumettre
			std::deque<ThreadSubmittedWork> _tasks ;
			// Scheduler parent
			Scheduler* _parentScheduler ;
			// Compteur de tasks pour les id
			std::atomic<size_t> _taskCounter ;
			// Abort appelé
			std::atomic<bool> _abortCalled ;
			
		public :
		
			// Constructeur, destructeur
			Thread () ;
			~Thread () ;

			// Getters
			ThreadId getThreadId () const ;

			// Setters
			void setParentScheduler (Scheduler* value) ;

			// Contrôles Tasks
			ThreadSubmittedWork addTask (Task* task) ;
			bool cancelTask (const ThreadSubmittedWork& submittedWork) ;

			// Abort demandé
			void askAbort () ;
			void join () ;
			void askAbortAndJoin () ;

			// Le run
			void run () ;
	} ;
}