﻿//////////////////////////////////////////////
// TabWindow.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkWinUi
{
	class Window ;
}

/// Includes ---------------------------------

// nkWinUi
#include "Component.h"

#include "../Dll/DllDefines.h"

#include "../Inputs/CallbackRepository.h"

// Standards
#include <map>
#include <string>
#include <unordered_map>

/// Class ------------------------------------
	
namespace nkWinUi
{
	class DLL_WINUI_EXPORT TabWindow : public Component
	{
		public :

			// Tab class to keep needed info
			class DLL_WINUI_EXPORT Tab
			{
				protected :
						
					// Le titre
					std::string _label ;
					// La Window associee
					Window* _associatedWin ;
					// Son id
					int _id ;

					// La tab window parente
					TabWindow* _parentWindow ;

				public :

					// Constructeur, destructeur
					Tab () ;
					virtual ~Tab () ;

					// Getters
					const char* getLabel () const ;
					Window* getAssociatedWin () const ;
					int getId () const ;
					TabWindow* getParentWindow () const ;

					// Setters
					virtual void setLabel (const std::string_view& label, bool autoUpdate = true) ;
					void setAssociatedWin (Window* win) ;
					void setId (int id) ;
					void setParentWindow (TabWindow* parent) ;					
			} ;

		protected :

			// Functions
			virtual Tab* _allocateTab () const = 0 ;

		protected :
		
			// Attributes
			// Tabs associated to their label and id
			std::map<std::string, Tab*, std::less<>> _tabMemory ;
			std::map<int, Tab*> _indicesMemory ;
			// Also as a list in order
			std::vector<Tab*> _tabs ;

			// Active tab
			Tab* _activeTab ;

			// Callback when changing tab
			std::function<bool (TabWindow*, TabWindow::Tab*, bool)> _tabSwitchCallback ;
			std::function<bool (TabWindow*, const ClickEventData&)> _clickCallback ;
		
		public :
		
			// Constructor, destructor
			TabWindow () ;
			~TabWindow () ;

			// For tabs
			virtual Tab* addTab (const std::string_view& label) ;
			Tab* getTabByLabel (const std::string_view& label) const ;
			Tab* getTabById (int id) const ;
			Tab* getTabByIndex (unsigned int index) const ;
			virtual Tab* getTabByMouseRel (const nkMaths::Point& mouseRel) const = 0 ;
			int getTabIndex (Tab* tab) const ;
			unsigned int getTabCount () const ;
			virtual void reorderTab (Tab* tab, unsigned int newIndex) ;
			virtual void changeTabLabel (const std::string_view& oldLabel, const std::string_view& newLabel) ;
			virtual void eraseTab (Tab* tab) ;

			// Concerning layouts
			virtual Layout* setLayout (LAYOUT_TYPE layoutType) override ;

			// Active tab
			virtual void setActiveTab (Tab* toActivate) ;
			Tab* getActiveTab () const ;

			// Sizing inside
			virtual void adaptToDisplayArea (Window* toAdapt) const = 0 ;

			// Callbacks
			void setTabSwitchCallback (std::function<bool (TabWindow*, TabWindow::Tab*, bool)> value) ;
			void setClickCallback (std::function<bool (TabWindow*, const ClickEventData&)> value) ;
			const std::function<bool (TabWindow*, TabWindow::Tab*, bool)>& getTabSwitchCallback () const ;
			const std::function<bool (TabWindow*, const ClickEventData&)>& getClickCallback () const ;

			// Import / Export
			virtual void exportIntrospection (nkExport::Node* rootNode) override ;
			void importClassFromTree (nkExport::Node* rootNode) override ;
	} ;
}